/*************************************************************************\
* Copyright (C) 2016-2020 Dirk Zimoch
* Copyright (C) 2020-2025 European Spallation Source, ERIC
* SPDX-License-Identifier: EPICS
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution.
\*************************************************************************/

#ifndef INC_afterIocRunning_H
#define INC_afterIocRunning_H

void afterIocRunningRegister(void);

#endif /* INC_afterIocRunning_H */
